package geniusweb.bidspace;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import geniusweb.issuevalue.Bid;
import geniusweb.issuevalue.Domain;
import geniusweb.issuevalue.Value;
import tudelft.utilities.immutablelist.AbstractImmutableList;
import tudelft.utilities.immutablelist.ImmutableList;
import tudelft.utilities.immutablelist.Outer;

/**
 * A list containing all complete bids in the space. This is an
 * {@link ImmutableList} so it can contain all bids without pre-computing them.
 * 
 */
public class AllBidsList extends AbstractImmutableList<Bid> {

	private final List<String> issues;
	private final Outer<? extends Value> allValuePermutations;

	/**
	 * A list containing all bids in the space. This is an ImmutableList so it
	 * can contain all bids without pre-computing them.
	 * 
	 */
	public AllBidsList(Domain domain) {
		if (domain == null)
			throw new IllegalArgumentException("domain=null");
		issues = new ArrayList<>(domain.getIssues());

		List<ImmutableList<Value>> values = issues.stream()
				.map(issue -> domain.getValues(issue))
				.collect(Collectors.toList());

		allValuePermutations = new Outer<Value>(values);
	}

	@Override
	public Bid get(BigInteger index) {
		ImmutableList<? extends Value> nextValues = allValuePermutations
				.get(index);

		Map<String, Value> issueValues = new HashMap<>();
		for (int n = 0; n < issues.size(); n++) {
			issueValues.put(issues.get(n),
					nextValues.get(BigInteger.valueOf(n)));
		}
		return new Bid(issueValues);
	}

	@Override
	public BigInteger size() {
		return allValuePermutations.size();
	}

}
